 /****************************************************************************\
*  RTC Demo                                                                    *
*  Copyright 2004 Timmy Brolin                                                 *
*                                                                              *
*  This program is free software; you can redistribute it and/or modify        *
*  it under the terms of the GNU General Public License as published by        *
*  the Free Software Foundation; either version 2 of the License, or           *
*  (at your option) any later version.                                         *
*                                                                              *
*  This program is distributed in the hope that it will be useful,             *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
*  GNU General Public License for more details.                                *
*                                                                              *
*  You should have received a copy of the GNU General Public License           *
*  along with this program; if not, write to the Free Software                 *
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   *
 \****************************************************************************/

// This demo is intended to show how to access the RTC (Real Time Clock) in certain 
// Gameboy Advance cartridges, such as Pokemon cartridges, the X-ROM flash cartridge,
// and the F2A Ultra flash cartridge.
// To try this demo on a Pokemon cartridge, simply put the Pokemon cartridge in your 
// Gameboy Advance and multiboot this demo.

// This demo does not work with EZ RTC flash cartridges, simply because the RTC in 
// thoose cartridges is non-standard and incompatible. It is currently unknown how
// to access the EZ RTC.

#include <time.h>

// Send commands to RTC
void rtc_command(char cmd){
	int foobar;

	*(volatile unsigned short*)0x80000C6=7;  // write
	*(volatile unsigned short*)0x80000C4=3;  //CS=0, IO=1, SCK=1
	*(volatile unsigned short*)0x80000C4=7;  //CS=1, IO=1, SCK=1

	for(foobar=0; foobar<8; foobar++){
		*(volatile unsigned short*)0x80000C4=4|((cmd>>6)&2);
		*(volatile unsigned short*)0x80000C4=(cmd>>6)|5;
		cmd<<=1;
	}
}

// Reads data serially from RTC
char rtc_read(void){
    char rtcbyte=0;
	int foobar;
	*(volatile unsigned short*)0x80000C6=5;  // read
	for(foobar=0; foobar<8; foobar++){
		*(volatile unsigned short*)0x80000C4=4;
		rtcbyte>>=1;
		rtcbyte|=((*(volatile unsigned short*)0x80000C4)<<6)&0x80;
		*(volatile unsigned short*)0x80000C4=5;
	}
	return rtcbyte;
}

//Detect and enable RTC if found
int rtc_detect(void){
	*(volatile unsigned short*)0x80000C8=0;
	if(((*(volatile unsigned short*)0x80000C8)&1)==0){
		*(volatile unsigned short*)0x80000C8=1; //Try enabling RTC
		if(((*(volatile unsigned short*)0x80000C8)&1)==1)
			return 1; //RTC detected and enabled
	}
	return 0; //No RTC detected
}

//Init data transfer from RTC
void rtc_init_transfer(void){
	*(volatile unsigned short*)0x80000C4=3;
	rtc_command(0x65);
}

//End data transfer from RTC
void rtc_end_transfer(void){
	*(volatile unsigned short*)0x80000C8=1;
}
